/*****************************************************************************
* File Name: InterruptRoutines.c
*
* Version: 2.0
*
* Description:
* For project B_PICU for application note AN54460. Contains user ISRs (as
* opposed to auto-generated ISRs) for PICU and timer tick. The PICU ISR
* initiates a debounce timeout. The tick ISR waits for timeout, checks input
* switch status, and toggles LEDs.
*
* Related Document:
* Application note AN54460, PSoC 3/5LP Interrupts
*
* Hardware Dependency:
* Designed for PSoC 3, or PSoC 5LP. Designed for CY8CKIT-001 but can
* be easily adapted to work with any kit.
*
* Code Tested With:
* 1. PSoC Creator 3.0 SP1
*
******************************************************************************
* Copyright (2009 - 2013), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/

#include "InterruptRoutines.h"

/* register bit masks for the switches */
#define  SW1_MASK  1
#define  SW2_MASK  2

#define  TIMED_OUT  0

/* switch debounce timeout variables decremented by Tick_ISR(),
   and monitored and reset by PICU_ISR()
*/
static volatile uint8 CYDATA switch_1_timeout;
static volatile uint8 CYDATA switch_2_timeout;


/*****************************************************************************
* Function Name: Tick_ISR
******************************************************************************
* Summary: Interrupt once per millisecond. If timeout, toggle LED.
*
* Parameters: none; it's an ISR function
*
* Return: none; it's an ISR function
*
*****************************************************************************/
CY_ISR(Tick_ISR)
{
	if(switch_1_timeout != 0)
	{
		/* Come here if an edge was detected by the PICU.
		   Decrement and check timeout.
		*/
		if(--switch_1_timeout == TIMED_OUT)
		{
			/* Timed out. Toggle LED only on a switch press (= 0). */
			if((Switches_Read() & SW1_MASK) == 0)
			{
				LED1_Write(~LED1_Read()); /* toggle the LED */
			}
		}
	}

	/* repeat for switch 2 */
	if(switch_2_timeout != 0)
	{
		if(--switch_2_timeout == TIMED_OUT)
		{
			if((Switches_Read() & SW2_MASK) == 0)
			{
				LED2_Write(~LED2_Read()); /* toggle the LED */
			}
		}
	}
}


/*****************************************************************************
* Function Name: PICU_ISR
******************************************************************************
* Summary: Interrupt only occurs on falling edge of either pin, i.e., when
*		   switch is pressed. Checks and initiates debounce timeout.
*
* Parameters: none; it's an ISR function
*
* Return: none; it's an ISR function
*
*****************************************************************************/
CY_ISR(PICU_ISR)
{
	/* copies of PICU registers */
	uint8 CYDATA temp_stat;
	
	/* read the PICU interrupt status register, with a clear on read */
	temp_stat = Switches_ClearInterrupt();

	/* Process the PICU event on SW1 only if any
	   ongoing debounce period has timed out */
	if(((temp_stat & SW1_MASK) != 0) && (switch_1_timeout == TIMED_OUT))
	{
		/* reset the debounce timer for this button */
		switch_1_timeout = DEBOUNCE_TIME; 
	}
	
	/* Process the PICU event on SW2 only if any
	   ongoing debounce period has timed out */
	if(((temp_stat & SW2_MASK) != 0) && (switch_2_timeout == TIMED_OUT))
	{
		/* reset the debounce timer for this button */
		switch_2_timeout = DEBOUNCE_TIME; 
	}
}

/* [] END OF FILE */
